/**
 * @file io_utils.c
 *
 * @copyright Copyright  (C)  2020 Moritz Hanke <hanke@dkrz.de>
 *                                 Thomas Jahns <jahns@dkrz.de>
 *
 * @version 1.0
 * @author Moritz Hanke <hanke@dkrz.de>
 *         Thomas Jahns <jahns@dkrz.de>
 */
/*
 * Keywords:
 * Maintainer: Moritz Hanke <hanke@dkrz.de>
 * URL: https://dkrz-sw.gitlab-pages.dkrz.de/yac/
 *
 * This file is part of YAC.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are  permitted provided that the following conditions are
 * met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * Neither the name of the DKRZ GmbH nor the names of its contributors
 * may be used to endorse or promote products derived from this software
 * without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "io_utils.h"
#include "math.h"

static int determine_io_proc_step(int comm_size) {

  int num_procs_per_node;

  // estimate the number of processes per node
  if (comm_size % 36 == 0) {
    num_procs_per_node = 36;
  } else if (comm_size % 24 == 0) {
    num_procs_per_node = 24;
  } else if (comm_size % 16 == 0) {
    num_procs_per_node = 16;
  } else if (comm_size % 12 == 0) {
    num_procs_per_node = 12;
  } else if (comm_size % 8 == 0) {
    num_procs_per_node = 8;
  } else if (comm_size % 6 == 0) {
    num_procs_per_node = 6;
  } else if (comm_size % 4 == 0) {
    num_procs_per_node = 4;
  } else if (comm_size % 2 == 0) {
    num_procs_per_node = 2;
  } else {
    num_procs_per_node = 1;
  }

  int num_nodes = comm_size / num_procs_per_node;
  int num_io_procs = MIN((unsigned)sqrt((float)num_nodes), 64);
  return ((num_nodes + num_io_procs - 1) / num_io_procs) * num_procs_per_node;
}

int yac_is_io_proc(
  int * io_proc_step_, MPI_Comm comm, int * num_io_procs, int * io_idx) {

  int comm_rank, comm_size;
  yac_mpi_call(MPI_Comm_rank(comm, &comm_rank), comm);
  yac_mpi_call(MPI_Comm_size(comm, &comm_size), comm);

  int io_proc_step;

  // determine io_proc_step
  if (*io_proc_step_ <= 0) {
    *io_proc_step_ = (io_proc_step = determine_io_proc_step(comm_size));
  } else {
    io_proc_step = *io_proc_step_;
  }

  // compute which procs does io
  *num_io_procs = (comm_size + io_proc_step - 1) / io_proc_step;
  *io_idx = comm_rank / io_proc_step;
  return !(comm_rank % io_proc_step);
}
