/**
 * @file yac_redirstdout.c
 *
 * @copyright Copyright  (C)  2014 Rene Redler <rene.redler@mpimet.mpg.de>
 *
 * @author Rene Redler <rene.redler@mpimet.mpg.de>
 */

/*
 * Keywords:
 * Maintainer: Rene Redler <rene.redler@mpimet.mpg.de>
 *
 * URL: https://dkrz-sw.gitlab-pages.dkrz.de/yac/
 *
 * This file is part of YAC.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are  permitted provided that the following conditions are
 * met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * Neither the name of the DKRZ GmbH nor the names of its contributors
 * may be used to endorse or promote products derived from this software
 * without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* ---------------------------------------------------------------------
  Date          Programmer   Description
  ------------  ----------   -----------
  01. Dec 2003  R. Redler    created
  01. Dec 2005  H. Ritzdorf  revised

   Copyright 2006-2010, NEC Europe Ltd., London, UK.
   All rights reserved. Use is subject to OASIS4 license terms.
   --------------------------------------------------------------------- */

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "yac_interface.h"
#include "utils.h"

void yac_redirstdout ( const char *filestem, int lenstr,
                       int parallel, int my_pe,
                       int npes, int *ierror ) {

        size_t len_alloc = lenstr;

        register int n_digits, num;
        register int redirect = 1;

        char *sname, *ename;
        register char *p_err, *p_std;

        ASSERT (lenstr > 0)
        ASSERT (my_pe  >= 0)

        *ierror = 0;

        /* get number of digits in rank */

        num = (npes > 0) ? npes : 1;
        n_digits = (int) (log10((double)(num) + (double) 0.5)) + 1;
        ASSERT (n_digits > 0)

        /* allocate file names */

#define NUMBER_OF_EXTENSIONS 2
#define LENGTH_OF_ERR 3

        len_alloc += n_digits + NUMBER_OF_EXTENSIONS + LENGTH_OF_ERR + 1;

        sname = (char *) xmalloc (len_alloc);
        ename = (char *) xmalloc (len_alloc);

        if (!sname || !ename) {
                fprintf (stderr, "PSMILe_redirstdout: Cannot allocate memory. %d bytes\n", (int) len_alloc);
                *ierror = 1;
                return;
        }

        /* copy base name to standard output and standard error file name */

        memcpy (sname, filestem, (size_t) lenstr);
        memcpy (ename, filestem, (size_t) lenstr);

        p_std = sname + (size_t) lenstr;
        p_err = ename + (size_t) lenstr;

        if (parallel == 1) {

                /* all processes write into their own file */

                sprintf (p_std, ".%0*d",     n_digits, my_pe);
                sprintf (p_err, ".err.%0*d", n_digits, my_pe);
        }

        else {

                /* only local root redirects stdout */

                redirect = (my_pe == 0);

                if( redirect ) {
                        sprintf (p_std, ".log");
                        sprintf (p_err, ".err");
                }
        }

        /* redirect stdout/stderr */

        if (redirect) {
                freopen (sname, "w", stdout);
                freopen (ename, "w", stderr);
        }

        /* free file names */

        free (ename);
        free (sname);
}

